#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from node.unl import get_unl_nodes, get_as_node_type

from blockchain.candidate_block.candidate_block_main import candidate_block


def GetCandidateBlocks():
    """
    Collects candidate blocks and candidate block hashes 
    from connected unl nodes and returns them in the 
    candidate_block class
    """    

    nodes = get_as_node_type(get_unl_nodes())

    the_candidate_blocks = []
    the_candidate_block_hashes = []

    for node in nodes:
        if node.candidate_block:
            already_in_list_candidate_blocks = False
            for other_blocks in the_candidate_blocks[:]:
                if other_blocks["signature"] == node.candidate_block["signature"]:
                    already_in_list_candidate_blocks = True
            if not already_in_list_candidate_blocks:
                the_candidate_blocks.append(node.candidate_block)
        if node.candidate_block_hash:
            already_in_list_candidate_block_hashes = False
            for other_block_hashes in the_candidate_block_hashes[:]:
                if (
                    other_block_hashes["signature"]
                    == node.candidate_block_hash["signature"]
                ):
                    already_in_list_candidate_block_hashes = True
            if not already_in_list_candidate_block_hashes:
                the_candidate_block_hashes.append(node.candidate_block_hash)

    return candidate_block(the_candidate_blocks, the_candidate_block_hashes)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

from kivymd.uix.screen import MDScreen

from kivy.uix.progressbar import ProgressBar
from kivy.clock import Clock




class DecentraWelcomeScreen(MDScreen):
 pb = ProgressBar()
 def __init__(self, *args, **kwargs):
    super(DecentraWelcomeScreen, self).__init__(*args, **kwargs)
    self.update_bar_trigger = Clock.schedule_interval(self.update_bar, 0.01)



 def update_bar(self, dt):
    if self.ids.pb.value < 100:
        self.ids.pb.value += 1
    else:
        self.update_bar_trigger.cancel()
        self.manager.current = "decentra root screen"


    




#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import os


from kivymd.uix.gridlayout import MDGridLayout
from kivymd.uix.screen import MDScreen
from kivymd.uix.button import MDFlatButton
from kivymd_extensions.sweetalert import SweetAlert
from kivymd.toast import toast

from kivy.core.clipboard import Clipboard

from node.node_connection import ndstart, ndstop, ndconnect, ndconnectmixdb, ndid

from lib.settings_system import the_settings
from lib.status import Status

from blockchain.block.get_block import GetBlockFromOtherNode
from blockchain.block.create_block import CreateBlock

class NodeScreen(MDScreen):
    pass

# Start Node Server
class start_node_server_Box(MDGridLayout):
    cols = 2
# End

# Add UNL Node
class add_unl_node_Box(MDGridLayout):
    cols = 2
# End

class NodeBox(MDGridLayout):
    cols = 2
    FONT_PATH = f"{os.environ['DECENTRA_ROOT']}/gui_lib/fonts/"

    # Start Node Server
    start_node_server_dialog = None
    def show_start_node_server_dialog(self):
        if not self.start_node_server_dialog:
            self.start_node_server_dialog = SweetAlert(
                title="Start Node Server",
                type="custom",
                auto_dismiss=False,
                content_cls=start_node_server_Box(),
                buttons=[
                    MDFlatButton(
                        text="CANCEL",
                        on_press=self.dismiss_start_node_server_dialog,
                        font_size= "18sp",
                        font_name= self.FONT_PATH + "RobotoCondensed-Bold",
                    ),
                    MDFlatButton(
                        text="OK",
                        on_press=self.start_node_server_func,
                        font_size= "18sp",
                        font_name= self.FONT_PATH + "RobotoCondensed-Bold",
                    ),
                ],
            )
        self.start_node_server_dialog.open()


    def get_start_node_server_dialog_text(self):
        text_list = []
        for obj in self.start_node_server_dialog.content_cls.children:
            for sub_obj in obj.children:
                text_list.append(sub_obj.text)
                sub_obj.text = ""

        return text_list  
    def start_node_server_func(self, widget):
        text_list = self.get_start_node_server_dialog_text()
        ip = text_list[1]
        port = text_list[0]

        print(ip)
        print(port)
        
        ndstart(ip, int(port))

        self.start_node_server_dialog.dismiss()
    def dismiss_start_node_server_dialog(self,widget):
        self.get_start_node_server_dialog_text()

        self.start_node_server_dialog.dismiss()


    def start_node_server(self):
        self.show_start_node_server_dialog()
    # End


    def stop_node_server(self):
        ndstop()
    def connect_to_know_node(self):
        ndconnectmixdb()


    # Connect a Node

    connect_a_node_dialog = None
    def show_connect_a_node_dialog(self):
        if not self.connect_a_node_dialog:
            self.connect_a_node_dialog = SweetAlert(
                title="Connect a Node",
                type="custom",
                auto_dismiss=False,
                content_cls=start_node_server_Box(),
                buttons=[
                    MDFlatButton(
                        text="CANCEL",
                        on_press=self.dismiss_connect_a_node_dialog,
                        font_size= "18sp",
                        font_name= self.FONT_PATH + "RobotoCondensed-Bold",
                    ),
                    MDFlatButton(
                        text="OK",
                        on_press=self.connect_a_node_func,
                        font_size= "18sp",
                        font_name= self.FONT_PATH + "RobotoCondensed-Bold",
                    ),
                ],
            )
        self.connect_a_node_dialog.open()


    def get_connect_a_node_dialog_text(self):
        text_list = []
        for obj in self.connect_a_node_dialog.content_cls.children:
            for sub_obj in obj.children:
                text_list.append(sub_obj.text)
                sub_obj.text = ""

        return text_list  
    def connect_a_node_func(self,widget):
        text_list = self.get_connect_a_node_dialog_text()
        ip = text_list[1]
        port = text_list[0]

        print(ip)
        print(port)
        
        ndconnect(ip, int(port))

        self.connect_a_node_dialog.dismiss()
    def dismiss_connect_a_node_dialog(self,widget):
        self.get_connect_a_node_dialog_text()

        self.connect_a_node_dialog.dismiss()

    def connect_a_node(self):
        self.show_connect_a_node_dialog()
    # End   

    # Add UNL Node

    add_unl_node_dialog = None
    def show_add_unl_node_dialog(self):
        if not self.add_unl_node_dialog:
            self.add_unl_node_dialog = SweetAlert(
                title="Add UNL Node",
                type="custom",
                auto_dismiss=False,
                content_cls=add_unl_node_Box(),
                buttons=[
                    MDFlatButton(
                        text="CANCEL",
                        on_press=self.dismiss_add_unl_node_dialog,
                        font_size= "18sp",
                        font_name= self.FONT_PATH + "RobotoCondensed-Bold",
                    ),
                    MDFlatButton(
                        text="OK",
                        on_press=self.add_unl_node_func,
                        font_size= "18sp",
                        font_name= self.FONT_PATH + "RobotoCondensed-Bold",
                    ),
                ],
            )
        self.add_unl_node_dialog.open()


    def get_add_unl_node_dialog_text(self):
        text_list = []
        for obj in self.add_unl_node_dialog.content_cls.children:
            for sub_obj in obj.children:
                text_list.append(sub_obj.text)
                sub_obj.text = ""

        return text_list  
    def add_unl_node_func(self,widget):
        text_list = self.get_add_unl_node_dialog_text()
        pubkey = text_list[0]

        print(pubkey)

        from node.unl import save_new_unl_node
        save_new_unl_node(pubkey)        

        self.add_unl_node_dialog.dismiss()
    def dismiss_add_unl_node_dialog(self,widget):
        self.get_add_unl_node_dialog_text()

        self.add_unl_node_dialog.dismiss()

    def add_unl_node(self):
        self.show_add_unl_node_dialog()
    # End
    def get_block(self):
        if the_settings()["test_mode"]:
            CreateBlock()
        else:
            GetBlockFromOtherNode()

    def nd_id(self):
        Clipboard.copy(ndid())
        SweetAlert().fire(
            "The ID has been copied to your clipboard.",
            type='success',
        )

    def status(self):
        toast("Calculating...")
        status = Status()
        if status == "Good":
            SweetAlert().fire(
                "Good",
                type='success',
            )
        elif status == "Not bad":
            SweetAlert().fire(
                "Not bad",
                type='info'
            )
        elif status == "Bad":
            SweetAlert().fire(
                "Not bad",
                type='question'
            )
        elif status == "Very bad":
            SweetAlert().fire(
                "Very bad",
                type='warning'
            )
        elif status == "Not work":
            SweetAlert().fire(
                "Not work",
                type='failure'
            )



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import pickle
import os

from lib.config_system import get_config
from config import MY_TRANSACTION_PATH


def SavetoMyTransaction(tx):
    """
    Saves the transaction to the transaction db.
    """

    currently_list = GetMyTransaction()
    currently_list.append(tx)

    os.chdir(get_config()["main_folder"])
    with open(MY_TRANSACTION_PATH, "wb") as my_transaction_file:
        pickle.dump(currently_list, my_transaction_file, protocol=2)


def GetMyTransaction():
    """
    Returns the transaction db.
    """

    os.chdir(get_config()["main_folder"])

    if not os.path.exists(MY_TRANSACTION_PATH):
        return []

    
    with open(MY_TRANSACTION_PATH, "rb") as my_transaction_file:
        obj = pickle.load(my_transaction_file)
        if not len(obj) == 1:
            obj.remove(obj[0])
        return obj



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from wallet.wallet import Address

from accounts.account import GetAccounts


def GetSequanceNumber(user, block):
    user = Address(user)
    sequance_number = 0
    for Accounts in GetAccounts():

        if Accounts.Address == user:

            sequance_number = Accounts.sequance_number

            for trans in block.pendingTransaction + block.validating_list:
                if user == trans.fromUser:
                    sequance_number += 1

            return sequance_number
    return sequance_number



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from lib.settings_system import the_settings
from lib.mixlib import dprint

from node.myownp2pn import mynode

from blockchain.block.block_main import Block
from blockchain.block.get_block import GetBlock

from wallet.wallet import Wallet_Import


def CreateBlock():
    """
    If test mode is on, creates genesis block
    and send the connected nodes, if it is off,
    it calls get_block() function.
    """

    if the_settings()["test_mode"]:
        dprint("Creating the genesis block")
        Block(0, Wallet_Import(-1, 3))
        mynode.main_node.send_full_accounts()
        mynode.main_node.send_full_chain()
        mynode.main_node.send_full_blockshash()
    else:
        dprint("Getting block from nodes")
        GetBlock()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

import os

from kivy import Config
Config.set('graphics', 'width', '700')
Config.set('graphics', 'height', '450')
Config.set('graphics', 'minimum_width', '700')
Config.set('graphics', 'minimum_height', '450')
Config.set('input', 'mouse', 'mouse,disable_multitouch')

from kivy.lang import Builder

from kivymd.app import MDApp

from lib.config_system import get_config
os.environ["DECENTRA_ROOT"] = get_config()["main_folder"]


KV_DIR = f"{os.environ['DECENTRA_ROOT']}/gui_lib/libs/kv/"

for kv_file in os.listdir(KV_DIR):
    with open(os.path.join(KV_DIR, kv_file), encoding="utf-8") as kv:
        Builder.load_string(kv.read())

KV = """
#:import FadeTransition kivy.uix.screenmanager.FadeTransition
#:import DecentraWelcomeScreen gui_lib.libs.baseclass.welcome_screen.DecentraWelcomeScreen
#:import DecentraRootScreen gui_lib.libs.baseclass.root_screen.DecentraRootScreen

ScreenManager:
    transition: FadeTransition()

    DecentraWelcomeScreen:
        name: "decentra register screen"

    DecentraRootScreen:
        name: "decentra root screen"

"""


class GUI(MDApp):
    """
    An MDApp based  GUI class.
    GUI class consists of 2 elements:
      * title: Title of the app.
      * icon: icon of the app.
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.title = "Decentra Network"
        self.icon = f"{os.environ['DECENTRA_ROOT']}/gui_lib/images/logo.ico"


    def build(self):
        """
        Some configurations.
        """

        self.theme_cls.primary_palette = "Green"
        FONT_PATH = f"{os.environ['DECENTRA_ROOT']}/gui_lib/fonts/"

        self.theme_cls.font_styles.update(
            {
                "H1": [FONT_PATH + "RobotoCondensed-Light", 96, False, -1.5],
                "H2": [FONT_PATH + "RobotoCondensed-Light", 60, False, -0.5],
                "H3": [FONT_PATH + "Eczar-Regular", 48, False, 0],
                "H4": [FONT_PATH + "RobotoCondensed-Regular", 34, False, 0.25],
                "H5": [FONT_PATH + "RobotoCondensed-Regular", 24, False, 0],
                "H6": [FONT_PATH + "RobotoCondensed-Bold", 20, False, 0.15],
                "Subtitle1": [
                    FONT_PATH + "RobotoCondensed-Regular",
                    16,
                    False,
                    0.15,
                ],
                "Subtitle2": [
                    FONT_PATH + "RobotoCondensed-Medium",
                    14,
                    False,
                    0.1,
                ],
                "Body1": [FONT_PATH + "Eczar-Regular", 16, False, 0.5],
                "Body2": [FONT_PATH + "RobotoCondensed-Light", 14, False, 0.25],
                "Button": [FONT_PATH + "RobotoCondensed-Bold", 14, True, 1.25],
                "Caption": [
                    FONT_PATH + "RobotoCondensed-Regular",
                    12,
                    False,
                    0.4,
                ],
                "Overline": [
                    FONT_PATH + "RobotoCondensed-Regular",
                    10,
                    True,
                    1.5,
                ],
            }
        )
        
        return Builder.load_string(KV)

def start():
    GUI().run()

if __name__ == '__main__':
    start()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import json
import os
from lib.config_system import get_config

from config import *


def save_settings(new_settings):
    """
    Saves the settings.
    """

    os.chdir(get_config()["main_folder"])
    with open(SETTING_PATH, "w") as settings_file:
        json.dump(new_settings, settings_file, indent=4)


def create_and_save_the_settings(test_mode_settings=False, debug_mode_settings=False):
    """
    Creates and saves settings.
    """

    temp_json = {}
    temp_json["test_mode"] = test_mode_settings

    temp_json["debug_mode"] = debug_mode_settings

    temp_json["wallet"] = 0

    save_settings(temp_json)
    return temp_json


def test_mode(new_value):
    """
    Changes the test_mode setting.

    Inputs:
      * new_value: New value for the test_mode
    """

    settings = the_settings()
    settings["test_mode"] = new_value
    save_settings(settings)


def debug_mode(new_value):
    """
    Changes the debug_mode setting.

    Inputs:
      * new_value: New value for the debug_mode
    """

    settings = the_settings()
    settings["debug_mode"] = new_value
    save_settings(settings)


def change_wallet(new_value):
    """
    Changes the debug_mode setting.

    Inputs:
      * new_value: New value for the debug_mode
    """

    settings = the_settings()
    settings["wallet"] = new_value
    save_settings(settings)


def the_settings():
    """
    Returns the settings. If it doesn't exist, it creates,
    saves and returns.
    """

    os.chdir(get_config()["main_folder"])

    if not os.path.exists(SETTING_PATH):
        return create_and_save_the_settings()
    else:
        with open(SETTING_PATH, "rb") as settings_file:
            return json.load(settings_file)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from wallet.wallet import Address

from accounts.account import Account
from accounts.account import save_accounts, GetAccounts


def ProccesstheTransaction(block):
    """
    It performs the transactions in the block.vali list and
    puts the transactions in order.

    Queuing is required so that all nodes have the same transaction hash.
    """

    from_user_list = []
    temp_validating_list = block.validating_list

    temp_accounts = GetAccounts()

    for trans in block.validating_list:
        touser_inlist = False
        address_of_fromUser = Address(trans.fromUser)

        for Accounts in temp_accounts:

            if Accounts.Address == address_of_fromUser:
                Accounts.balance -= float(trans.amount) + trans.transaction_fee
                Accounts.sequance_number += 1
                from_user_list.append(Accounts)
                block.edited_accounts.append(Accounts)

            elif Accounts.Address == trans.toUser:
                Accounts.balance += float(trans.amount)
                touser_inlist = True
                block.edited_accounts.append(Accounts)

        # If not included in the temp_accounts, add.
        if not touser_inlist:
            temp_accounts.append(Account(trans.toUser, float(trans.amount)))

    # Converts public keys to an Account class to use when ordering
    temp_pubkeys = []
    for tx_item in temp_validating_list[:]:
        for Account_item in from_user_list:
            if Address(str(tx_item.fromUser)) == Account_item.Address:
                temp_pubkeys.append(str(tx_item.fromUser))
                tx_item.fromUser = Account_item

    # Orders the transactions by Address index of temp_accounts.
    temp_validating_list = sorted(
        temp_validating_list, key=lambda x: temp_accounts.index(x.fromUser)
    )

    # Converts the Account class to Public key.
    for temp_validating_list_item in temp_validating_list[:]:
        for temp_pubkey in temp_pubkeys:
            if str(temp_validating_list_item.fromUser) == Address(temp_pubkey):
                temp_validating_list_item.fromUser = temp_pubkey

    # Syncs new sorted list to block.validating_list
    block.validating_list = temp_validating_list

    save_accounts(temp_accounts)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

import unittest


class Test_Wallet(unittest.TestCase):

    def test_saving_and_importing_and_deleting_the_wallet(self):

        password = "123"

        temp_private_key = Wallet_Create(password)

        saved_wallets = get_saved_wallet()

        result = False
        for each_wallet in saved_wallets:
            if temp_private_key == (saved_wallets[each_wallet]["privatekey"]):
                if temp_private_key == (Wallet_Import(each_wallet, 1, password)):
                
                    Wallet_Delete(each_wallet)
                    result = True if each_wallet not in get_saved_wallet() else False
                    break
                elif decrypt(temp_private_key, password) == (Wallet_Import(each_wallet, 1, password)):
                    Wallet_Delete(each_wallet)
                    result = True if each_wallet not in get_saved_wallet() else False
                    break

        self.assertEqual(result, True, "A problem on the saving and importing the wallet.")

    def test_Private_Pem_Conversion(self):

        password = "123"

        temp_private_key_class = Wallet_Create(password, save=False)
        pem = temp_private_key_class.toPem()
        privateKey2 = PrivateKey.fromPem(pem)
        self.assertEqual(temp_private_key_class.secret, privateKey2.secret)
        self.assertEqual(temp_private_key_class.curve, privateKey2.curve)

    def test_Public_Conversion(self):

        password = "123"

        privateKey = Wallet_Create(password, save=False)
        publicKey1 = privateKey.publicKey()
        pem = publicKey1.toPem()
        publicKey2 = PublicKey.fromPem(pem)
        self.assertEqual(publicKey1.point.x, publicKey2.point.x)
        self.assertEqual(publicKey1.point.y, publicKey2.point.y)
        self.assertEqual(publicKey1.curve, publicKey2.curve)



import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), "..","src"))
from wallet.wallet import Wallet_Create, get_saved_wallet, Wallet_Import, Wallet_Delete, PrivateKey, toBytes, PublicKey
from lib.encryption import decrypt
unittest.main(exit=False)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import time

from lib.mixlib import dprint

from consensus.consensus_first_round import consensus_round_1
from consensus.consensus_second_round import consensus_round_2

from blockchain.block.get_block import GetBlock


def consensus_trigger():
    """
    Consensus process consists of 2 stages. This function makes
    the necessary redirects according to the situation and works
    to shorten the block time.
    """

    dprint("Consensus Trigger")
    block = GetBlock()

    if block.validated:
        true_time = (
            block.block_time_change_time
            + block.block_time
            + (
                (block.block_time_change_block - block.sequance_number)
                * block.block_time
            )
        )
        if block.newly:
            true_time -= 1
        if not int(time.time()) < true_time:
            block.newly = False
            block.reset_the_block()
    else:
        if block.raund_1_starting_time is None:
            block.raund_1_starting_time = int(time.time())
        if not block.raund_1:

            consensus_round_1(block)
        elif not block.raund_2:
            consensus_round_2(block)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import pickle
import os

from lib.config_system import get_config

from config import TEMP_BLOCKSHASH_PATH, TEMP_BLOCKSHASH_PART_PATH


def SaveBlockshash(the_blockshash):
    os.chdir(get_config()["main_folder"])
    with open(TEMP_BLOCKSHASH_PATH, "wb") as block_file:
        pickle.dump(the_blockshash, block_file, protocol=2)


def SaveBlockshash_part(the_blockshash):
    os.chdir(get_config()["main_folder"])
    with open(TEMP_BLOCKSHASH_PART_PATH, "wb") as block_file:
        pickle.dump(the_blockshash, block_file, protocol=2)


def GetBlockshash():
    os.chdir(get_config()["main_folder"])
    if not os.path.exists(TEMP_BLOCKSHASH_PATH):
        return []
    else:
        with open(TEMP_BLOCKSHASH_PATH, "rb") as block_file:
            return pickle.load(block_file)


def GetBlockshash_part():
    os.chdir(get_config()["main_folder"])
    if not os.path.exists(TEMP_BLOCKSHASH_PART_PATH):
        return []
    else:
        with open(TEMP_BLOCKSHASH_PART_PATH, "rb") as block_file:
            return pickle.load(block_file)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

from node.node import *

from wallet.wallet import Signature , Ecdsa , PublicKey , PrivateKey , Wallet_Import

from lib.mixlib import dprint

from hashlib import sha256


from lib.merkle_root import MerkleTree


from transactions.transaction import Transaction

import os

from config import TEMP_BLOCK_PATH, LOADING_BLOCK_PATH, TEMP_ACCOUNTS_PATH, TEMP_BLOCKSHASH_PATH

from blockchain.block.get_block import GetBlock

class mynode (Node):
    main_node = None
    unl_nodes = []

    def __init__(self, host, port):
        self.__class__.main_node = self
        super(mynode, self).__init__(host, port, None)
        print("MyPeer2PeerNode: Started")



    def message_from_node(self, node, data):
        from node.unl import get_unl_nodes, get_as_node_type
    
        if str(data) == "sendmefullblock":
            self.send_full_chain(node)
        print("Data Type: "+str(type(data))+"\n")


        try:
            from node.unl import node_is_unl
            if data["fullblock"] == 1 and node_is_unl(node.id) and Ecdsa.verify("fullblock"+data["byte"], Signature.fromBase64(data["signature"]), PublicKey.fromPem(node.id)):
                print("getting chain")
                self.get_full_chain(data,node)
        except Exception as e:
            print(e)

        try:
            from node.unl import node_is_unl
            if data["fullaccounts"] == 1 and node_is_unl(node.id) and Ecdsa.verify("fullaccounts"+data["byte"], Signature.fromBase64(data["signature"]), PublicKey.fromPem(node.id)):
                print("getting chain")
                self.get_full_accounts(data,node)
        except Exception as e:
            print(e)

        try:
            from node.unl import node_is_unl
            if data["fullblockshash"] == 1 and node_is_unl(node.id) and Ecdsa.verify("fullblockshash"+data["byte"], Signature.fromBase64(data["signature"]), PublicKey.fromPem(node.id)):
                self.get_full_blockshash(data,node)
        except Exception as e:
            print(e)

        try:
         if data["transactionrequest"]  == 1:
            self.get_transaction(data,node)
        except Exception as e:
            print(e)



            
        try:
         if data["action"]  == "myblock":
            self.get_candidate_block(data,node)
        except Exception as e:
            print(e)




        try:
         if data["action"]  == "myblockhash":
            self.get_candidate_block_hash(data,node)
        except Exception as e:
            print(e)


        print("message_from_node from " + node.id + ": " + str(data))
        
    def node_disconnect_to_outbound_node(self, node):
        print("node wants to disconnect with oher outbound node: " + node.id)
        
    def node_request_to_stop(self):
        print("node is requested to stop!")




    def send_my_block(self,nodes):
         system = GetBlock()

         new_list = []

         signature_list = []

         for element in system.validating_list:
             new_list.append(element.dump_json())
             signature_list.append(element.signature)

         dprint("signature_list: "+str(signature_list))
         dprint("publickey from pem: "+str(Wallet_Import(0,1)))


         Merkle_signature_list = MerkleTree(signature_list).getRootHash() if len(signature_list) != 0 else "0"


         dprint("\nmerkleroot: "+Merkle_signature_list)

         data = {
             "action":"myblock",
             "transaction": new_list,
             "sequance_number": system.sequance_number,
             "signature":Ecdsa.sign("myblock"+Merkle_signature_list+str(system.sequance_number), PrivateKey.fromPem(Wallet_Import(0,1))).toBase64()
         }

         for each_node in nodes:
            dprint("Raund 1: second ok of get candidate block: "+ str(each_node.__dict__))
            self.send_data_to_node(each_node,data)

    def send_my_block_hash(self,nodes):
        system = GetBlock()

        if system.raund_1 and not system.raund_2:


         data = {
             "action":"myblockhash",
             "hash": system.hash,
             "sequance_number": system.sequance_number,
             "signature":Ecdsa.sign("myblockhash"+system.hash+str(system.sequance_number), PrivateKey.fromPem(Wallet_Import(0,1))).toBase64()
         }

         for each_node in nodes:
             dprint("Raund 2: second ok of get candidate block hashes: "+ str(each_node.__dict__))
             self.send_data_to_node(each_node,data)


    def get_candidate_block(self,data,node):

      dprint("Getting the candidate block")
      from node.unl import node_is_unl
      if node_is_unl(node.id) and GetBlock().sequance_number == data["sequance_number"]:
            dprint("is unl")

            signature_list = []
            for element in data["transaction"]:
                signature_list.append(element["signature"])

            dprint("signature_list: "+str(signature_list))

            merkle_root_of_signature_list = MerkleTree(signature_list).getRootHash() if len(signature_list) != 0 else "0"


            

            dprint("signatureverify: "+str(Ecdsa.verify("myblock"+merkle_root_of_signature_list, Signature.fromBase64(data["signature"]), PublicKey.fromPem(node.id))))
            dprint("publickey from pem: "+str(node.id))
            
            dprint("merkleroot: "+merkle_root_of_signature_list)

            if Ecdsa.verify("myblock"+merkle_root_of_signature_list+str(data["sequance_number"]), Signature.fromBase64(data["signature"]), PublicKey.fromPem(node.id)):
                dprint("ecdsa true")

                temp_tx = []

                for element in data["transaction"]:
                    temp_tx.append(Transaction.load_json(element))

                data["transaction"] = temp_tx

                node.candidate_block = data


    def get_candidate_block_hash(self,data,node):

      dprint("Getting the candidate block hash")

      from node.unl import node_is_unl
      if node_is_unl(node.id) and GetBlock().sequance_number == data["sequance_number"]:
            dprint("is unl")


            if Ecdsa.verify("myblockhash"+data["hash"]+str(data["sequance_number"]), Signature.fromBase64(data["signature"]), PublicKey.fromPem(node.id)):
                dprint("ecdsa true")
                data["sender"] = node.id

                node.candidate_block_hash = data



    def send_full_chain(self,node = None):
        dprint("Sending full chain to node or nodes."+" Node: "+ str(node))
        file = open(TEMP_BLOCK_PATH, "rb")
        SendData = file.read(1024)
        while SendData:

            data = {"fullblock" : 1,"byte" : (SendData.decode(encoding='iso-8859-1')),"signature" : Ecdsa.sign("fullblock"+str((SendData.decode(encoding='iso-8859-1'))), PrivateKey.fromPem(Wallet_Import(0,1))).toBase64()}
            if not node is None:
                self.send_data_to_node(node,data)
            else:
                self.send_data_to_nodes(data)

            SendData = file.read(1024) 

            dprint(SendData)

            if not SendData:
                data = {"fullblock" : 1,"byte" : "end","signature": Ecdsa.sign("fullblock"+"end", PrivateKey.fromPem(Wallet_Import(0,1))).toBase64()}
                if not node is None:
                    self.send_data_to_node(node,data)
                else:
                    self.send_data_to_nodes(data)


    def send_full_accounts(self,node = None):
        dprint("Sending full chain to node or nodes."+" Node: "+ str(node))
        file = open(TEMP_ACCOUNTS_PATH, "rb")
        SendData = file.read(1024)
        while SendData:

            data = {"fullaccounts" : 1,"byte" : (SendData.decode(encoding='iso-8859-1')),"signature" : Ecdsa.sign("fullaccounts"+str((SendData.decode(encoding='iso-8859-1'))), PrivateKey.fromPem(Wallet_Import(0,1))).toBase64()}
            if not node is None:
                self.send_data_to_node(node,data)
            else:
                self.send_data_to_nodes(data)

            SendData = file.read(1024) 

            dprint(SendData)

            if not SendData:
                data = {"fullaccounts" : 1,"byte" : "end","signature": Ecdsa.sign("fullaccounts"+"end", PrivateKey.fromPem(Wallet_Import(0,1))).toBase64()}
                if not node is None:
                    self.send_data_to_node(node,data)
                else:
                    self.send_data_to_nodes(data)

    def send_full_blockshash(self,node = None):
        dprint("Sending full chain to node or nodes."+" Node: "+ str(node))
        file = open(TEMP_BLOCKSHASH_PATH, "rb")
        SendData = file.read(1024)
        while SendData:

            data = {"fullblockshash" : 1,"byte" : (SendData.decode(encoding='iso-8859-1')),"signature" : Ecdsa.sign("fullblockshash"+str((SendData.decode(encoding='iso-8859-1'))), PrivateKey.fromPem(Wallet_Import(0,1))).toBase64()}
            if not node is None:
                self.send_data_to_node(node,data)
            else:
                self.send_data_to_nodes(data)

            SendData = file.read(1024) 

            dprint(SendData)

            if not SendData:
                data = {"fullblockshash" : 1,"byte" : "end","signature": Ecdsa.sign("fullblockshash"+"end", PrivateKey.fromPem(Wallet_Import(0,1))).toBase64()}
                if not node is None:
                    self.send_data_to_node(node,data)
                else:
                    self.send_data_to_nodes(data)


    def get_full_chain(self,data,node):
      
      get_ok = False

      if not os.path.exists(TEMP_BLOCK_PATH):
        get_ok = True
      else:
        system = GetBlock()
        if node.id == system.dowload_true_block:
            get_ok = True

      
      if get_ok:



        if str(data["byte"]) == "end":

            os.rename(LOADING_BLOCK_PATH, TEMP_BLOCK_PATH)
            
            from blockchain.block.block_main import apps_starter
            from consensus.consensus_main import consensus_trigger
            from lib.perpetualtimer import perpetualTimer
            from app.app_main import apps_starter
            system = GetBlock()
            system.newly = True
            system.change_transaction_fee()

            system.exclude_validators = []
            dprint(system.sequance_number)
            perpetualTimer(system.consensus_timer,consensus_trigger).start()
            apps_starter()
            system.save_block()
            


        else:
            file = open(LOADING_BLOCK_PATH, "ab")
                
            file.write((data["byte"].encode(encoding='iso-8859-1')))
            file.close()


    def get_full_blockshash(self,data,node):
      
      get_ok = False

      if not os.path.exists(TEMP_BLOCKSHASH_PATH):
        get_ok = True
      else:
        system = GetBlock()
        if node.id == system.dowload_true_block:
            get_ok = True

      
      if get_ok:
        file = open(TEMP_BLOCKSHASH_PATH, "ab")
                
        file.write((data["byte"].encode(encoding='iso-8859-1')))
        file.close()

    def get_full_accounts(self,data,node):
      
      get_ok = False

      if not os.path.exists(TEMP_ACCOUNTS_PATH):
        get_ok = True
      else:
        system = GetBlock()
        if node.id == system.dowload_true_block:
            get_ok = True

      
      if get_ok:
        file = open(TEMP_ACCOUNTS_PATH, "ab")
                
        file.write((data["byte"].encode(encoding='iso-8859-1')))
        file.close()



    def get_transaction(self,data,node):
        dprint("Getting the transactions")
        system = GetBlock()
        system.createTrans(sequance_number = data["sequance_number"],signature =data["signature"],fromUser = data["fromUser"],toUser = data["to_user"],data = data["data"],amount = data["amount"],transaction_fee = data["transaction_fee"],transaction_sender=node, transaction_time = data["transaction_time"])



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import time

from blockchain.block.get_block import GetBlock


def Status():
    first_block = GetBlock()
    start_time = time.time()
    while True:
        time.sleep(5)
        new_time = time.time()
        new_block = GetBlock()
        difference = int(new_time - start_time) 
        if not (first_block.sequance_number + first_block.empty_block_number) == (new_block.sequance_number + new_block.empty_block_number):
            if difference <= 6:
                return "Good"
            elif difference <= 10:
                return "Not bad"
            elif difference <= 20:
                return "Bad"
            elif difference >= 20:
                return "Very bad"
        else:
            if difference >= 20:
                return "Not work"



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import time

from lib.mixlib import dprint

from node.unl import get_as_node_type, get_unl_nodes
from node.myownp2pn import mynode

from blockchain.candidate_block.get_candidate_blocks import GetCandidateBlocks
from blockchain.block.calculate_hash import CalculateHash

from transactions.process_the_transaction import ProccesstheTransaction


def consensus_round_1(block):
        """
        At this stage of the consensus process,
        The transactions of our and the unl nodes 
        are evaluated and transactions accepted by 
        owned by more than 50 percent.
        Inputs:
          * block: The block (class) we want consensus 
          round 1 to be done
        """

        unl_nodes = get_unl_nodes()
        if not block.raund_1_node:
              dprint("Raund 1: in get candidate blocks\n")

 
              mynode.main_node.send_my_block(get_as_node_type(unl_nodes))
              block.raund_1_node = True
              block.save_block()
        candidate_class = GetCandidateBlocks()
        dprint("Raund 1 Conditions")
        dprint(len(candidate_class.candidate_blocks) > ((len(unl_nodes) * 80)/100))
        dprint((int(time.time()) - block.raund_1_starting_time) < block.raund_1_time)
        if len(candidate_class.candidate_blocks) > ((len(unl_nodes) * 80)/100):

         if not (int(time.time()) - block.raund_1_starting_time) < block.raund_1_time:
          temp_validating_list = []
          dprint("Raund 1: first ok")
          dprint(len(candidate_class.candidate_blocks))
          for candidate_block in candidate_class.candidate_blocks[:]:
              print(candidate_block)


              for other_block_tx in candidate_block["transaction"]:

                  tx_valid = 0

                  for my_txs in block.validating_list:
                      if other_block_tx.signature == my_txs.signature:
                          tx_valid += 1


                  if len(candidate_class.candidate_blocks) != 1:
                      dprint("Raund 1: Test tx")
                      for other_block in candidate_class.candidate_blocks[:]:
                         if candidate_block["signature"] != other_block["signature"]:
                            dprint("Raund 1: Test tx 2")
                            for other_block_txs in other_block["transaction"]:
                                if other_block_tx.signature == other_block_txs.signature:
                                    dprint("Raund 1: Test tx 3")
                                    tx_valid += 1
                  else:
                      tx_valid += 1


                  if tx_valid > (len(unl_nodes) / 2):
                      dprint("Raund 1: second ok")
                      already_in_ok = False
                      for alrady_tx  in temp_validating_list[:]:
                          
                          if other_block_tx.signature == alrady_tx.signature:
                              already_in_ok = True
                      if not already_in_ok:
                            dprint("Raund 1: third ok")
                            temp_validating_list.append(other_block_tx)




        
          newly_added_list = []
                
          for my_validating_list in block.validating_list[:]:
              ok = False
              for my_temp_validating_list in temp_validating_list[:]:
                  if my_validating_list.signature == my_temp_validating_list.signature:
                      ok = True
              block.validating_list.remove(my_validating_list) 
              if not ok:
                newly_added_list.append(my_validating_list)
                

                
          block.validating_list = temp_validating_list
        
          
          for each_newly in newly_added_list:
                block.createTrans(each_newly.sequance_number, each_newly.signature, each_newly.fromUser, each_newly.toUser, each_newly.transaction_fee, each_newly.data, each_newly.amount, transaction_sender = None, transaction_time = each_newly.time)

          block.raund_1 = True

          block.raund_2_starting_time = int(time.time())

          



          ProccesstheTransaction(block)


          block.hash = CalculateHash(block)



          block.save_block()
        
                

         else:
            if not block.decrease_the_time == 3:
                block.decrease_the_time += 1
                block.increase_the_time = 0
                block.save_block()

        else:
            if not (int(time.time()) - block.raund_1_starting_time) < block.raund_1_time:
                if not block.increase_the_time == 3:
                    block.increase_the_time += 1
                    block.decrease_the_time = 0                
                    block.save_block()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

from kivy.properties import ColorProperty, StringProperty

from kivymd.theming import ThemableBehavior
from kivymd.uix.behaviors import RectangularRippleBehavior
from kivymd.uix.boxlayout import MDBoxLayout
from kivymd.uix.screen import MDScreen


class DecentraRootScreen(MDScreen):
    pass


class DecentraListItem(ThemableBehavior, RectangularRippleBehavior, MDBoxLayout):
    text = StringProperty()
    secondary_text = StringProperty()
    tertiary_text = StringProperty()
    bar_color = ColorProperty((1, 0, 0, 1))


class DecentraSeeAllButton(RectangularRippleBehavior, MDBoxLayout):
    pass



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2021 Decentra Network Developers
Copyright (c) 2018 Stark Bank S.A.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from hashlib import sha256


from sys import version_info as pyVersion
from binascii import hexlify, unhexlify




if pyVersion.major == 3:
    # py3 constants and conversion functions

    xrange = range
    stringTypes = (str,) # lgtm [py/multiple-definition]
    intTypes = (int, float)

    def toString(string):
        return string.decode("latin-1")

    def toBytes(string):
        return string.encode("latin-1")

    def safeBinaryFromHex(hexString):
        return unhexlify(hexString)

    def safeHexFromBinary(byteString):
        return hexlify(byteString)
else:
    # py2 constants and conversion functions

    stringTypes = (str, unicode) # lgtm [py/multiple-definition]
    intTypes = (int, float, long)

    def toString(string):
        return string

    def toBytes(string):
        return string

    def safeBinaryFromHex(hexString):
        return unhexlify(hexString)

    def safeHexFromBinary(byteString):
        return hexlify(byteString)


class BinaryAscii:

    @classmethod
    def hexFromBinary(cls, data):
        """
        Return the hexadecimal representation of the binary data. Every byte of data is converted into the
        corresponding 2-digit hex representation. The resulting string is therefore twice as long as the length of data.
        :param data: binary
        :return: hexadecimal string
        """
        return safeHexFromBinary(data)

    @classmethod
    def binaryFromHex(cls, data):
        """
        Return the binary data represented by the hexadecimal string hexstr. This function is the inverse of b2a_hex().
        hexstr must contain an even number of hexadecimal digits (which can be upper or lower case), otherwise a TypeError is raised.
        :param data: hexadecimal string
        :return: binary
        """
        return safeBinaryFromHex(data)

    @classmethod
    def numberFromString(cls, string):
        """
        Get a number representation of a string
        :param String to be converted in a number
        :return: Number in hex from string
        """
        return int(cls.hexFromBinary(string), 16)

    @classmethod
    def stringFromNumber(cls, number, length):
        """
        Get a string representation of a number
        :param number to be converted in a string
        :param length max number of character for the string
        :return: hexadecimal string
        """

        fmtStr = "%0" + str(2 * length) + "x"
        return toString(cls.binaryFromHex((fmtStr % number).encode()))

from random import SystemRandom


class RandomInteger:

    @classmethod
    def between(cls, min, max):
        """
        Return integer x in the range: min <= x <= max
        :param min: minimum value of the integer
        :param max: maximum value of the integer
        :return:
        """

        return SystemRandom().randrange(min, max + 1)

from sys import version_info as pyVersion
from binascii import hexlify, unhexlify


if pyVersion.major == 3:
    # py3 constants and conversion functions

    xrange = range
    stringTypes = (str,)
    intTypes = (int, float)

    def toString(string):
        return string.decode("latin-1")

    def toBytes(string):
        return string.encode("latin-1")

    def safeBinaryFromHex(hexString):
        return unhexlify(hexString)

    def safeHexFromBinary(byteString):
        return hexlify(byteString)
else:
    # py2 constants and conversion functions

    stringTypes = (str, unicode)
    intTypes = (int, float, long)

    def toString(string):
        return string

    def toBytes(string):
        return string

    def safeBinaryFromHex(hexString):
        return unhexlify(hexString)

    def safeHexFromBinary(byteString):
        return hexlify(byteString)


class Ecdsa:

    @classmethod
    def sign(cls, message, privateKey, hashfunc=sha256):
        hashMessage = hashfunc(toBytes(message)).digest()
        numberMessage = BinaryAscii.numberFromString(hashMessage)
        curve = privateKey.curve

        r, s, randSignPoint = 0, 0, None
        while r == 0 or s == 0:
            randNum = RandomInteger.between(1, curve.N - 1)
            randSignPoint = Math.multiply(curve.G, n=randNum, A=curve.A, P=curve.P, N=curve.N)
            r = randSignPoint.x % curve.N
            s = ((numberMessage + r * privateKey.secret) * (Math.inv(randNum, curve.N))) % curve.N
        recoveryId = randSignPoint.y & 1
        if randSignPoint.y > curve.N:
            recoveryId += 2

        return Signature(r=r, s=s, recoveryId=recoveryId)

    @classmethod
    def verify(cls, message, signature, publicKey, hashfunc=sha256):
        hashMessage = hashfunc(toBytes(message)).digest()
        numberMessage = BinaryAscii.numberFromString(hashMessage)
        curve = publicKey.curve
        sigR = signature.r
        sigS = signature.s
        inv = Math.inv(sigS, curve.N)
        u1 = Math.multiply(curve.G, n=(numberMessage * inv) % curve.N, A=curve.A, P=curve.P, N=curve.N)
        u2 = Math.multiply(publicKey.point, n=(sigR * inv) % curve.N, A=curve.A, P=curve.P, N=curve.N)


        #add = Math.add(u1, u2, P=curve.P, A=curve.A) # test and delete
        #return sigR == add.x # test and delete
        
        add = Math.add(u1, u2, P=curve.P, A=curve.A)
        modX = add.x % curve.N
        return sigR == modX



from base64 import b64encode, b64decode


class Base64:

    @classmethod
    def decode(cls, string):
        return b64decode(string)

    @classmethod
    def encode(cls, string):
        return b64encode(string)



hexAt = "\x00"
hexB = "\x02"
hexC = "\x03"
hexD = "\x04"
hexF = "\x06"
hex0 = "\x30"

hex31 = 0x1f
hex127 = 0x7f
hex129 = 0xa0
hex160 = 0x80
hex224 = 0xe0

bytesHex0 = toBytes(hex0)
bytesHexB = toBytes(hexB)
bytesHexC = toBytes(hexC)
bytesHexD = toBytes(hexD)
bytesHexF = toBytes(hexF)


def encodeSequence(*encodedPieces):
    totalLengthLen = sum([len(p) for p in encodedPieces])
    return hex0 + _encodeLength(totalLengthLen) + "".join(encodedPieces)


def encodeInteger(x):
    assert x >= 0
    t = ("%x" % x).encode()

    if len(t) % 2:
        t = toBytes("0") + t

    x = BinaryAscii.binaryFromHex(t)
    num = x[0] if isinstance(x[0], intTypes) else ord(x[0])

    if num <= hex127:
        return hexB + chr(len(x)) + toString(x)
    return hexB + chr(len(x) + 1) + hexAt + toString(x)


def encodeOid(first, second, *pieces):
    assert first <= 2
    assert second <= 39

    encodedPieces = [chr(40 * first + second)] + [_encodeNumber(p) for p in pieces]
    body = "".join(encodedPieces)

    return hexF + _encodeLength(len(body)) + body


def encodeBitString(t):
    return hexC + _encodeLength(len(t)) + t


def encodeOctetString(t):
    return hexD + _encodeLength(len(t)) + t


def encodeConstructed(tag, value):
    return chr(hex129 + tag) + _encodeLength(len(value)) + value


def removeSequence(string):
    _checkSequenceError(string=string, start=bytesHex0, expected="30")

    length, lengthLen = _readLength(string[1:])
    endSeq = 1 + lengthLen + length

    return string[1 + lengthLen: endSeq], string[endSeq:]


def removeInteger(string):
    _checkSequenceError(string=string, start=bytesHexB, expected="02")

    length, lengthLen = _readLength(string[1:])
    numberBytes = string[1 + lengthLen:1 + lengthLen + length]
    rest = string[1 + lengthLen + length:]
    nBytes = numberBytes[0] if isinstance(
        numberBytes[0], intTypes
    ) else ord(numberBytes[0])

    assert nBytes < hex160

    return int(BinaryAscii.hexFromBinary(numberBytes), 16), rest


def removeObject(string):
    _checkSequenceError(string=string, start=bytesHexF, expected="06")

    length, lengthLen = _readLength(string[1:])
    body = string[1 + lengthLen:1 + lengthLen + length]
    rest = string[1 + lengthLen + length:]
    numbers = []

    while body:
        n, lengthLength = _readNumber(body)
        numbers.append(n)
        body = body[lengthLength:]

    n0 = numbers.pop(0)
    first = n0 // 40
    second = n0 - (40 * first)
    numbers.insert(0, first)
    numbers.insert(1, second)

    return tuple(numbers), rest


def removeBitString(string):
    _checkSequenceError(string=string, start=bytesHexC, expected="03")

    length, lengthLen = _readLength(string[1:])
    body = string[1 + lengthLen:1 + lengthLen + length]
    rest = string[1 + lengthLen + length:]

    return body, rest


def removeOctetString(string):
    _checkSequenceError(string=string, start=bytesHexD, expected="04")

    length, lengthLen = _readLength(string[1:])
    body = string[1 + lengthLen:1 + lengthLen + length]
    rest = string[1 + lengthLen + length:]

    return body, rest


def removeConstructed(string):
    s0 = _extractFirstInt(string)
    if (s0 & hex224) != hex129:
        raise Exception("wanted constructed tag (0xa0-0xbf), got 0x%02x" % s0)

    tag = s0 & hex31
    length, lengthLen = _readLength(string[1:])
    body = string[1 + lengthLen:1 + lengthLen + length]
    rest = string[1 + lengthLen + length:]

    return tag, body, rest


def fromPem(pem):
    t = "".join([
        l.strip() for l in pem.splitlines()
        if l and not l.startswith("-----")
    ])
    return Base64.decode(t)


def toPem(der, name):
    b64 = toString(Base64.encode(der))
    lines = ["-----BEGIN " + name + "-----\n"]
    lines.extend([
        b64[start:start + 64] 
        for start in xrange(0, len(b64), 64)
    ])
    lines.append("\n")
    lines.append("-----END " + name + "-----\n")

    return "".join(lines)


def _encodeLength(length):
    assert length >= 0

    if length < hex160:
        return chr(length)

    s = ("%x" % length).encode()
    if len(s) % 2:
        s = "0" + s

    s = BinaryAscii.binaryFromHex(s)
    lengthLen = len(s)

    return chr(hex160 | lengthLen) + str(s)


def _encodeNumber(n):
    b128Digits = []
    while n:
        b128Digits.insert(0, (n & hex127) | hex160)
        n >>= 7

    if not b128Digits:
        b128Digits.append(0)

    b128Digits[-1] &= hex127

    return "".join([chr(d) for d in b128Digits])


def _readLength(string):
    num = _extractFirstInt(string)
    if not (num & hex160):
        return (num & hex127), 1

    lengthLen = num & hex127

    if lengthLen > len(string) - 1:
        raise Exception("ran out of length bytes")

    return int(BinaryAscii.hexFromBinary(string[1:1 + lengthLen]), 16), 1 + lengthLen


def _readNumber(string):
    number = 0
    lengthLen = 0
    while True:
        if lengthLen > len(string):
            raise Exception("ran out of length bytes")

        number <<= 7
        d = string[lengthLen]
        if not isinstance(d, intTypes):
            d = ord(d)

        number += (d & hex127)
        lengthLen += 1
        if not d & hex160:
            break

    return number, lengthLen


def _checkSequenceError(string, start, expected):
    if not string.startswith(start):
        raise Exception(
            "wanted sequence (0x%s), got 0x%02x" %
            (expected, _extractFirstInt(string))
        )


def _extractFirstInt(string):
    return string[0] if isinstance(string[0], intTypes) else ord(string[0])

#
# Elliptic Curve Equation
#
# y^2 = x^3 + A*x + B (mod P)
#

class Point:

    def __init__(self, x=0, y=0, z=0):
        self.x = x
        self.y = y
        self.z = z


class CurveFp:

    def __init__(self, A, B, P, N, Gx, Gy, name, oid, nistName=None):
        self.A = A
        self.B = B
        self.P = P
        self.N = N
        self.G = Point(Gx, Gy)
        self.name = name
        self.nistName = nistName
        self.oid = oid

    def contains(self, p):
        """
        Verify if the point `p` is on the curve
        :param p: Point p = Point(x, y)
        :return: boolean
        """
        return (p.y**2 - (p.x**3 + self.A * p.x + self.B)) % self.P == 0

    def length(self):
        return (1 + len("%x" % self.N)) // 2


secp256k1 = CurveFp(
    name="secp256k1",
    A=0x0000000000000000000000000000000000000000000000000000000000000000,
    B=0x0000000000000000000000000000000000000000000000000000000000000007,
    P=0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffefffffc2f,
    N=0xfffffffffffffffffffffffffffffffebaaedce6af48a03bbfd25e8cd0364141,
    Gx=0x79be667ef9dcbbac55a06295ce870b07029bfcdb2dce28d959f2815b16f81798,
    Gy=0x483ada7726a3c4655da4fbfc0e1108a8fd17b448a68554199c47d08ffb10d4b8,
    oid=(1, 3, 132, 0, 10)
)

prime256v1 = CurveFp(
    name="prime256v1",
    nistName="P-256",
    A=0xffffffff00000001000000000000000000000000fffffffffffffffffffffffc,
    B=0x5ac635d8aa3a93e7b3ebbd55769886bc651d06b0cc53b0f63bce3c3e27d2604b,
    P=0xffffffff00000001000000000000000000000000ffffffffffffffffffffffff,
    N=0xffffffff00000000ffffffffffffffffbce6faada7179e84f3b9cac2fc632551,
    Gx=0x6b17d1f2e12c4247f8bce6e563a440f277037d812deb33a0f4a13945d898c296,
    Gy=0x4fe342e2fe1a7f9b8ee7eb4a7c0f9e162bce33576b315ececbb6406837bf51f5,
    oid=(1, 2, 840, 10045, 3, 1, 7),
)
p256 = prime256v1

supportedCurves = [
    secp256k1,
    prime256v1,
]

curvesByOid = {curve.oid: curve for curve in supportedCurves}


hexAt = "\x00"


class PrivateKey:

    def __init__(self, curve=secp256k1, secret=None):
        self.curve = curve
        self.secret = secret or RandomInteger.between(1, curve.N - 1)

    def publicKey(self):
        curve = self.curve
        publicPoint = Math.multiply(
            p=curve.G,
            n=self.secret,
            N=curve.N,
            A=curve.A,
            P=curve.P,
        )
        return PublicKey(point=publicPoint, curve=curve)

    def toString(self):
        return BinaryAscii.stringFromNumber(number=self.secret, length=self.curve.length())

    def toDer(self):
        encodedPublicKey = self.publicKey().toString(encoded=True)

        return encodeSequence(
            encodeInteger(1),
            encodeOctetString(self.toString()),
            encodeConstructed(0, encodeOid(*self.curve.oid)),
            encodeConstructed(1, encodeBitString(encodedPublicKey)),
        )

    def toPem(self):
        return toPem(der=toBytes(self.toDer()), name="EC PRIVATE KEY")

    @classmethod
    def fromPem(cls, string):
        privateKeyPem = string[string.index("-----BEGIN EC PRIVATE KEY-----"):]
        return cls.fromDer(fromPem(privateKeyPem))

    @classmethod
    def fromDer(cls, string):
        t, empty = removeSequence(string)
        if len(empty) != 0:
            raise Exception(
                "trailing junk after DER private key: " +
                BinaryAscii.hexFromBinary(empty)
            )

        one, t = removeInteger(t)
        if one != 1:
            raise Exception(
                "expected '1' at start of DER private key, got %d" % one
            )

        privateKeyStr, t = removeOctetString(t)
        tag, curveOidStr, t = removeConstructed(t)
        if tag != 0:
            raise Exception("expected tag 0 in DER private key, got %d" % tag)

        oidCurve, empty = removeObject(curveOidStr)

        if len(empty) != 0:
            raise Exception(
                "trailing junk after DER private key curve_oid: %s" %
                BinaryAscii.hexFromBinary(empty)
            )

        if oidCurve not in curvesByOid:
            raise Exception(
                "unknown curve with oid %s; The following are registered: %s" % (
                    oidCurve,
                    ", ".join([curve.name for curve in supportedCurves])
                )
            )

        curve = curvesByOid[oidCurve]

        if len(privateKeyStr) < curve.length():
            privateKeyStr = hexAt * (curve.lenght() - len(privateKeyStr)) + privateKeyStr

        return cls.fromString(privateKeyStr, curve)

    @classmethod
    def fromString(cls, string, curve=secp256k1):
        return PrivateKey(secret=BinaryAscii.numberFromString(string), curve=curve)


class Signature:

    def __init__(self, r, s, recoveryId=None):
        self.r = r
        self.s = s
        self.recoveryId = recoveryId

    def toDer(self, withRecoveryId=False):
        encodedSequence = encodeSequence(encodeInteger(self.r), encodeInteger(self.s))
        if not withRecoveryId:
            return encodedSequence
        return chr(27 + self.recoveryId) + encodedSequence

    def toBase64(self, withRecoveryId=False):
        return toString(Base64.encode(toBytes(self.toDer(withRecoveryId=withRecoveryId))))

    @classmethod
    def fromDer(cls, string, recoveryByte=False):
        recoveryId = None
        if recoveryByte:
            recoveryId = string[0] if isinstance(string[0], intTypes) else ord(string[0])
            recoveryId -= 27
            string = string[1:]

        rs, empty = removeSequence(string)
        if len(empty) != 0:
            raise Exception("trailing junk after DER signature: %s" % BinaryAscii.hexFromBinary(empty))

        r, rest = removeInteger(rs)
        s, empty = removeInteger(rest)
        if len(empty) != 0:
            raise Exception("trailing junk after DER numbers: %s" % BinaryAscii.hexFromBinary(empty))

        return Signature(r=r, s=s, recoveryId=recoveryId)

    @classmethod
    def fromBase64(cls, string, recoveryByte=False):
        der = Base64.decode(string)
        return cls.fromDer(der, recoveryByte)



class Math:

    @classmethod
    def multiply(cls, p, n, N, A, P):
        """
        Fast way to multily point and scalar in elliptic curves
        :param p: First Point to mutiply
        :param n: Scalar to mutiply
        :param N: Order of the elliptic curve
        :param P: Prime number in the module of the equation Y^2 = X^3 + A*X + B (mod p)
        :param A: Coefficient of the first-order term of the equation Y^2 = X^3 + A*X + B (mod p)
        :return: Point that represents the sum of First and Second Point
        """
        return cls._fromJacobian(
            cls._jacobianMultiply(
                cls._toJacobian(p),
                n,
                N,
                A,
                P,
            ),
            P,
        )

    @classmethod
    def add(cls, p, q, A, P):
        """
        Fast way to add two points in elliptic curves
        :param p: First Point you want to add
        :param q: Second Point you want to add
        :param P: Prime number in the module of the equation Y^2 = X^3 + A*X + B (mod p)
        :param A: Coefficient of the first-order term of the equation Y^2 = X^3 + A*X + B (mod p)
        :return: Point that represents the sum of First and Second Point
        """
        return cls._fromJacobian(
            cls._jacobianAdd(
                cls._toJacobian(p),
                cls._toJacobian(q),
                A,
                P,
            ),
            P,
        )

    @classmethod
    def inv(cls, x, n):
        """
        Extended Euclidean Algorithm. It's the 'division' in elliptic curves
        :param x: Divisor
        :param n: Mod for division
        :return: Value representing the division
        """
        if x == 0:
            return 0

        lm, hm = 1, 0
        low, high = x % n, n
        while low > 1:
            r = high // low
            nm, new = hm - lm * r, high - low * r
            lm, low, hm, high = nm, new, lm, low

        return lm % n

    @classmethod
    def _toJacobian(cls, p):
        """
        Convert point to Jacobian coordinates
        :param p: First Point you want to add
        :return: Point in Jacobian coordinates
        """
        return Point(p.x, p.y, 1)

    @classmethod
    def _fromJacobian(cls, p, P):
        """
        Convert point back from Jacobian coordinates
        :param p: First Point you want to add
        :param P: Prime number in the module of the equation Y^2 = X^3 + A*X + B (mod p)
        :return: Point in default coordinates
        """
        z = cls.inv(p.z, P)

        return Point(
            (p.x * z ** 2) % P,
            (p.y * z ** 3) % P,
        )

    @classmethod
    def _jacobianDouble(cls, p, A, P):
        """
        Double a point in elliptic curves
        :param p: Point you want to double
        :param P: Prime number in the module of the equation Y^2 = X^3 + A*X + B (mod p)
        :param A: Coefficient of the first-order term of the equation Y^2 = X^3 + A*X + B (mod p)
        :return: Point that represents the sum of First and Second Point
        """
        if not p.y:
            return Point(0, 0, 0)

        ysq = (p.y ** 2) % P
        S = (4 * p.x * ysq) % P
        M = (3 * p.x ** 2 + A * p.z ** 4) % P
        nx = (M**2 - 2 * S) % P
        ny = (M * (S - nx) - 8 * ysq ** 2) % P
        nz = (2 * p.y * p.z) % P
        return Point(nx, ny, nz)

    @classmethod
    def _jacobianAdd(cls, p, q, A, P):
        """
        Add two points in elliptic curves
        :param p: First Point you want to add
        :param q: Second Point you want to add
        :param P: Prime number in the module of the equation Y^2 = X^3 + A*X + B (mod p)
        :param A: Coefficient of the first-order term of the equation Y^2 = X^3 + A*X + B (mod p)
        :return: Point that represents the sum of First and Second Point
        """

        if not p.y:
            return q
        if not q.y:
            return p

        U1 = (p.x * q.z ** 2) % P
        U2 = (q.x * p.z ** 2) % P
        S1 = (p.y * q.z ** 3) % P
        S2 = (q.y * p.z ** 3) % P

        if U1 == U2:
            if S1 != S2:
                return Point(0, 0, 1)
            return cls._jacobianDouble(p, A, P)

        H = U2 - U1
        R = S2 - S1
        H2 = (H * H) % P
        H3 = (H * H2) % P
        U1H2 = (U1 * H2) % P
        nx = (R ** 2 - H3 - 2 * U1H2) % P
        ny = (R * (U1H2 - nx) - S1 * H3) % P
        nz = (H * p.z * q.z) % P

        return Point(nx, ny, nz)

    @classmethod
    def _jacobianMultiply(cls, p, n, N, A, P):
        """
        Multily point and scalar in elliptic curves
        :param p: First Point to mutiply
        :param n: Scalar to mutiply
        :param N: Order of the elliptic curve
        :param P: Prime number in the module of the equation Y^2 = X^3 + A*X + B (mod p)
        :param A: Coefficient of the first-order term of the equation Y^2 = X^3 + A*X + B (mod p)
        :return: Point that represents the sum of First and Second Point
        """
        if p.y == 0 or n == 0:
            return Point(0, 0, 1)

        if n == 1:
            return p

        if n < 0 or n >= N:
            return cls._jacobianMultiply(p, n % N, N, A, P)

        if (n % 2) == 0:
            return cls._jacobianDouble(
                cls._jacobianMultiply(
                    p,
                    n // 2,
                    N,
                    A,
                    P
                ),
                A,
                P,
            )

        # (n % 2) == 1:
        return cls._jacobianAdd(
            cls._jacobianDouble(
                cls._jacobianMultiply(
                    p,
                    n // 2,
                    N,
                    A,
                    P,
                ),
                A,
                P,
            ),
            p,
            A,
            P,
        )
class PublicKey:

    def __init__(self, point, curve):
        self.point = point
        self.curve = curve

    def toString(self, encoded=False):
        xString = BinaryAscii.stringFromNumber(
            number=self.point.x,
            length=self.curve.length(),
        )
        yString = BinaryAscii.stringFromNumber(
            number=self.point.y,
            length=self.curve.length(),
        )
        return "\x00\x04" + xString + yString if encoded else xString + yString

    def toDer(self):
        oidEcPublicKey = (1, 2, 840, 10045, 2, 1)
        encodeEcAndOid = encodeSequence(
            encodeOid(*oidEcPublicKey),
            encodeOid(*self.curve.oid),
        )

        return encodeSequence(encodeEcAndOid, encodeBitString(self.toString(encoded=True)))

    def toPem(self):
        return toPem(der=toBytes(self.toDer()), name="PUBLIC KEY")

    @classmethod
    def fromPem(cls, string):
        return cls.fromDer(fromPem(string))

    @classmethod
    def fromDer(cls, string):
        s1, empty = removeSequence(string)
        if len(empty) != 0:
            raise Exception("trailing junk after DER public key: {}".format(
                BinaryAscii.hexFromBinary(empty)
            ))

        s2, pointBitString = removeSequence(s1)

        oidPk, rest = removeObject(s2)

        oidCurve, empty = removeObject(rest)
        if len(empty) != 0:
            raise Exception("trailing junk after DER public key objects: {}".format(
                BinaryAscii.hexFromBinary(empty)
            ))

        if oidCurve not in curvesByOid:
            raise Exception(
                "Unknown curve with oid %s. Only the following are available: %s" % (
                    oidCurve,
                    ", ".join([curve.name for curve in supportedCurves])
                )
            )

        curve = curvesByOid[oidCurve]

        pointStr, empty = removeBitString(pointBitString)
        if len(empty) != 0:
            raise Exception(
                "trailing junk after public key point-string: " +
                BinaryAscii.hexFromBinary(empty)
            )

        return cls.fromString(pointStr[2:], curve)

    @classmethod
    def fromString(cls, string, curve=secp256k1, validatePoint=True):
        baseLen = curve.length()

        xs = string[:baseLen]
        ys = string[baseLen:]

        p = Point(
            x=BinaryAscii.numberFromString(xs),
            y=BinaryAscii.numberFromString(ys),
        )

        if validatePoint and not curve.contains(p):
            raise Exception(
                "point ({x},{y}) is not valid for curve {name}".format(
                    x=p.x, y=p.y, name=curve.name
                )
            )

        return PublicKey(point=p, curve=curve)




from config import *

from lib.config_system import get_config
from lib.settings_system import the_settings
from lib.encryption import encrypt, decrypt

import json
import os
from hashlib import sha256

def save_wallet_list(publicKey,privateKey,password):
    wallet_list = get_saved_wallet()


    wallet_list[publicKey] = {}

    wallet_list[publicKey]["publickey"] = publicKey
    wallet_list[publicKey]["privatekey"] = privateKey

    wallet_list[publicKey]["password_sha256"] = sha256(password.encode("utf-8")).hexdigest()


    



    os.chdir(get_config()["main_folder"])
    with open(WALLETS_PATH, 'w') as wallet_list_file:
        json.dump(wallet_list, wallet_list_file, indent=4)



def get_saved_wallet():
    
        os.chdir(get_config()["main_folder"])

        if not os.path.exists(WALLETS_PATH):
            return {}
        

        with open(WALLETS_PATH, 'rb') as wallet_list_file:
            return json.load(wallet_list_file)






def Wallet_Create(password, save = True):

    my_private_key = PrivateKey()
    my_public_key = my_private_key.publicKey()




    if save == True:
        encrypted_key = encrypt(my_private_key.toPem(),password) if not len(list(get_saved_wallet())) == 0 else my_private_key.toPem()
        del my_private_key
        save_wallet_list(my_public_key.toPem(),encrypted_key, password)
        return (encrypted_key)
    else:
        return (my_private_key)

def Wallet_Import(account,mode,password = None):
    """
    A function for get info about a wallet.

    Inputs:
      * account: Account index of saved accounts (if you give -1 the default wallet will use)
      * mode: Information mode [0 = Public key | 1 = Private key (needs password) | 2 = Returns sha256 of password | 3 = Returns the address of account]
      * password: Some function needed password for operation you can give with this input
    """

    temp_saved_wallet = get_saved_wallet()
    if isinstance(account,int):
        if not -1 == account:
            account = list(temp_saved_wallet)[account]
        else:
            account = list(temp_saved_wallet)[the_settings()["wallet"]]

    if mode == 0:
        my_public_key = temp_saved_wallet[account]["publickey"]

        return my_public_key
    elif mode == 1:
        if not password is None and not list(temp_saved_wallet).index(account) == 0:

            return decrypt(temp_saved_wallet[account]["privatekey"], password)
        else:
            my_private_key = temp_saved_wallet[account]["privatekey"]

            return my_private_key

    elif mode == 2:
            return temp_saved_wallet[account]["password_sha256"]

    elif mode == 3:
        my_address = temp_saved_wallet[account]["publickey"]
        my_address = "".join([
            l.strip() for l in my_address.splitlines()
            if l and not l.startswith("-----")
        ])
        my_address = Address(my_address)
        return my_address
    else:
        raise ValueError("the mode variable contains an unplanned value")


def Wallet_Delete(account):
    saved_wallet = get_saved_wallet()
    if account in saved_wallet:
        del saved_wallet[account]
        from lib.config_system import get_config
    
        os.chdir(get_config()["main_folder"])
        with open(WALLETS_PATH, 'w') as wallet_list_file:
            json.dump(saved_wallet, wallet_list_file, indent=4)

def Address(publickey):

    return sha256(sha256(publickey.encode("utf-8")).hexdigest().encode("utf-8")).hexdigest()[-40:]



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from transactions.send import send

from wallet.wallet import Wallet_Import


def send_coin(coin_amount, to_user, password):
    """
    A function for sending coins.

    Inputs:
      * coin_amount: A int or float amount to be sent.
      * to_user: Receiver's address.
      * password: Password of wallet.
    """

    my_public_key = Wallet_Import(-1, 0)
    my_private_key = Wallet_Import(-1, 1, password)

    if isinstance(coin_amount, int):
        coin_amount = float(coin_amount)

    if not isinstance(coin_amount, float):
        print("This is not int or float coin amount.")
        return None

    if coin_amount < 0:
        print("This is negative coin amount.")
        return None

    send(
        my_public_key=my_public_key,
        my_private_key=my_private_key,
        to_user=to_user,
        password=password,
        amount=coin_amount,
    )

    del my_private_key



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import time

from wallet.wallet import Ecdsa, PrivateKey

from accounts.get_sequance_number import GetSequanceNumber

from blockchain.block.get_block import GetBlock

from transactions.save_to_my_transaction import SavetoMyTransaction

def send(my_public_key, my_private_key, to_user, password, data=None, amount=None):
    """
    The main function for sending the transaction.

    Inputs:
      * my_public_key: Sender's public key.
      * my_private_key: Sender's private key.
      * to_user: Receiver's address.
      * data: A text that can be written into the transaction. (Can be None)
      * amount: A int or float amount to be sent. (Can be None)
    """

    my_public_key = "".join(
        [
            l.strip()
            for l in my_public_key.splitlines()
            if l and not l.startswith("-----")
        ]
    )

    system = GetBlock()
    sequance_number = GetSequanceNumber(my_public_key, system) + 1

    # Get the current fee
    transaction_fee = system.transaction_fee

    tx_time = int(time.time())

    the_tx = system.createTrans(
        sequance_number=sequance_number,
        signature=Ecdsa.sign(
            str(sequance_number)
            + str(my_public_key)
            + str(to_user)
            + str(data)
            + str(amount)
            + str(transaction_fee)
            + str(tx_time),
            PrivateKey.fromPem(my_private_key),
        ).toBase64(),
        fromUser=str(my_public_key),
        toUser=str(to_user),
        data=data,
        amount=amount,
        transaction_fee=transaction_fee,
        transaction_sender=None,
        transaction_time=tx_time,
    )

    SavetoMyTransaction(the_tx)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from wallet.wallet import Wallet_Import, get_saved_wallet, Wallet_Delete


from lib.settings_system import the_settings, change_wallet


def delete_current_wallet():
    """
    Deletes the current wallet.
    """

    if not the_settings()["wallet"] == 0:
        saved_wallets = get_saved_wallet()
        selected_wallet_pubkey = Wallet_Import(int(the_settings()["wallet"]), 0)
        for each_wallet in saved_wallets:
            if selected_wallet_pubkey == saved_wallets[each_wallet]["publickey"]:
                change_wallet(0)
                Wallet_Delete(each_wallet)
    else:
        print("First wallet cannot be deleted.")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import os

from threading import Thread

from lib.mixlib import dprint


class app(Thread):
    """
    It initiates the start functions of the applications in parallel
    from the main process.
    """

    def __init__(self, import_command, func):
        Thread.__init__(self)
        self.import_command = import_command
        self.func = func

    def run(self):
        exec(self.import_command)
        exec(self.func)
        dprint("App is started")


def apps_starter():
    """
    Finds applications and sends them to the app().
    """
    for folder_entry in os.scandir("app"):
        if (
            ".md" not in folder_entry.name
            and "__" not in folder_entry.name
            and "app_main" not in folder_entry.name
        ):
            for entry in os.scandir("app/apps/" + folder_entry.name):
                if entry.is_file():
                    if (
                        entry.name[0] != "_"
                        and ".py" in entry.name
                        and "_main" in entry.name
                    ):
                        import_command = f"from app.apps.{folder_entry.name}.{entry.name.replace('.py','')} import {entry.name.replace('.py','')}_run"
                        tx_command = f"{entry.name.replace('.py','')}_run()"

                        exec(import_command)

                        x = app(import_command, tx_command)
                        x.start()


def app_tigger(block):
    """
    Notifies applications of validated transactions after
    the block is validated.
    """
    for folder_entry in os.scandir("app"):
        if (
            ".md" not in folder_entry.name
            and "__" not in folder_entry.name
            and "app_main" not in folder_entry.name
        ):
            for entry in os.scandir("app/apps/" + folder_entry.name):
                if entry.is_file():
                    if (
                        entry.name[0] != "_"
                        and ".py" in entry.name
                        and "_main" in entry.name
                    ):
                        for (
                            trans
                        ) in block.validating_list:  # lgtm [py/unused-loop-variable]
                            import_command = f"from app.apps.{folder_entry.name}.{entry.name.replace('.py','')} import {entry.name.replace('.py','')}_tx"
                            tx_command = f"{entry.name.replace('.py','')}_tx(trans)"
                            exec(import_command)
                            exec(tx_command)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import json
import hashlib


class Transaction:
    """
    The transaction class.

    Transaction class consists of 8 elements:
      * sequance_number: A number from the account class that ensures that
      each transaction is valid once.
      * signature: A signature proving that the sender approved the transaction.
      * fromUser: Sender's public key.
      * toUser: Receiver's address.
      * data: A text that can be written into the transaction.
      * amount: A int or float amount to be sent.
      * transaction_fee: Fee for transaction.
      * time: Sending time.
    """

    def __init__(self, sequance_number, signature, fromUser, toUser, data, amount, transaction_fee, time_of_transaction):
        self.sequance_number = sequance_number
        self.signature = signature
        self.fromUser = fromUser
        self.toUser = toUser
        self.data = data
        self.amount = amount
        self.transaction_fee = transaction_fee
        self.time = time_of_transaction

    def dump_json(self):
        """
        Returns a json containing the account's data.
        """

        data = {
            "sequance_number": self.sequance_number,
            "signature": self.signature,
            "fromUser": self.fromUser,
            "toUser": self.toUser,
            "data": self.data,
            "amount": self.amount,
            "transaction_fee": self.transaction_fee,
            "transaction_time": self.time
        }
        return data

    def get_hash(self, encoding="ascii"):
        """
        Returns a sha256 created using the dump_json() function.
        """

        transaction_data = json.dumps(self.dump_json()).encode(encoding)
        return hashlib.sha256(transaction_data).hexdigest()

    @staticmethod
    def load_json(data):
        """
        Returns the json data received with dump_json() as an object again.
        """

        return Transaction(
            data['sequance_number'],
            data['signature'],
            data['fromUser'],
            data['toUser'],
            data['data'],
            data['amount'],
            data['transaction_fee'],
            data['transaction_time'],
            )
